      SUBROUTINE SUAER
      implicit none
C
C**** *SUAER*   - INITIALIZE COMMON YOMAER
C
C     PURPOSE.
C     --------
C           INITIALIZE YOMAER, THE COMMON THAT CONTAINS THE
C           RADIATIVE CHARACTERISTICS OF THE AEROSOLS
c
c     
c
C
C**   INTERFACE.
C     ----------
C        *CALL* *SUAER*
C
C        EXPLICIT ARGUMENTS :
C        --------------------
C        NONE
C
C        IMPLICIT ARGUMENTS :
C        --------------------
C        COMMON YOMAER
C
C     METHOD.
C     -------
C        SEE DOCUMENTATION
C
C     EXTERNALS.
C     ----------
C
C     REFERENCE.
C     ----------
C        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE
C     "IN CORE MODEL"
C
C     AUTHOR.
C     -------
C        JEAN-JACQUES MORCRETTE *ECMWF*
C
C     MODIFICATIONS.
C     --------------
C        ORIGINAL : 88-02-15
c        Frederic Hourdin
c        Richard Fournier (1996) Francois Forget (1996)
C     ------------------------------------------------------------------
C
C-----------------------------------------------------------------------
C
#include "dimensions.h"
#include "dimphys.h"
#include "dimradmars.h"
#include "yomaer.h"
C
C-----------------------------------------------------------------------
c
      INTEGER isun,iir,itra
      REAL tsun
      REAL longref,epref
      REAL longsun(nsun+1)
      REAL tsol
      REAL longir(nir+1)
      REAL epir(nir),omegir(nir),gir(nir)
      REAL t9s9

c     Aerosol Spectral properties :
#include "aerdata.h"

c   quelques initialisations a 0

      do itra=1,ntra
         do isun=1,nsun
            cga(isun,itra)=0.
            piza(isun,itra)=0.
            taua(isun,itra)=0.
         enddo
         do iir=1,nir
            caer(iir,itra)=0.
         enddo
      enddo
c
C-----------------------------------------------------------------------
C
C      ----------------------------------------------------------------
C
C*       1.    SHORTWAVE COEFFICIENTS
C              ----------------------
C
c
c on utilise le premier traceur uniquement
c
c._. a peine comprehensible mais :
c._.
c    1) on commence par calculer les proprietes optiques
c       moyennes sur les deux bandes solaires.
c       les epaisseurs optiques moyennes sont "en fait" des
c       $\overbar{\tau} \over \tau_{0.67micron}$ 
c._.
c
      tsun=6000.E+0
      longsun(1)=long1vis
      longsun(2)=long2vis
      longsun(3)=long3vis
      longref=longrefvis
      epref=1.E+0
      CALL aerave ( ndatavis,
     & longdatavis,epdatavis,omegdatavis,gdatavis,
     & longref,epref,tsun
     &             ,nsun,longsun,taua,piza,cga              )
c
C      ----------------------------------------------------------------
C
C*       2.    LONGWAVE COEFFICIENTS
C              ---------------------
C
c
c._.
c    2) puis on calcule les proprietes optiques moyennes
c       sur la bande IRTM a 9 micron.
c       l' epaisseur optique moyenne est "en fait"
c       $\overbar{\tau} \over \tau_{9micron}$ 
c._.
c
      tsol=215.D+0
      longir(1)=8.3E-6
      longir(2)=9.7E-6
      longref=longrefir
      epref=1.E+0
      CALL aerave ( ndatair,
     & longdatair,epdatair,omegdatair,gdatair,
     & longref,epref,tsol
     &             ,1,longir,epir,omegir,gir )
      t9s9=epir(1)
c
c._.
c    3) puis on calcule les proprietes optiques moyennes
c       sur les trois bandes IR.
c       les epaisseurs optiques moyennes sont "en fait"
c       et "dans un premier temps" des
c       $\overbar{\tau} \over \tau_{9micron}$ 
c._.
c
      tsol=215.D+0
      longir(1)=long1ir
      longir(2)=long1co2
      longir(3)=long2co2
      longir(4)=long2ir
      longref=longrefir
      epref=1.E+0
      CALL aerave ( ndatair,
     & longdatair,epdatair,omegdatair,gdatair,
     & longref,epref,tsol
     &             ,nir-1,longir,epir,omegir,gir )
c
c._.
c    4) enfin il reste a ecrire les epaisseurs optiques moyennes
c       en
c       $\overbar{\tau} \over \tau_{0.67micron}$ 
c._.
c
      DO iir=1,nir-1
        epir(iir)=epir(iir)/t9s9/solsir
      ENDDO
c
c
c
c
c co2
c
      qextaer(1)=epir(2)
      qextaer(2)=epir(2)
      omegaer(1)=omegir(2)
      omegaer(2)=omegir(2)
      gaer(1)=gir(2)
      gaer(2)=gir(2)
c
c bande 9 micron
c
      qextaer(3)=epir(1)
      omegaer(3)=omegir(1)
      gaer(3)=gir(1)
c
c grandes longueurs d'ondes
c
      DO iir=4,nir
        qextaer(iir)=epir(iir-1)
        omegaer(iir)=omegir(iir-1)
        gaer(iir)=gir(iir-1)
      ENDDO
c
      PRINT*,'Rapport Solaire/IR :',solsir
      PRINT*

      PRINT *,'Les donnees spectrales :'
      PRINT *,'Solaire ---->'
      PRINT *,'Qext / Qext0.67 ; omega ; g'
      DO isun=1,nsun
c._.
c            taua(isun,1)=1.E+0
c            piza(isun,1)=0.86E+0
c            cga(isun,1)=0.79E+0
c._.
        PRINT *,taua(isun,1),piza(isun,1),cga(isun,1)
      ENDDO
      PRINT *,'IR ---->'
      PRINT *,'Qext / Qext0.67 ; omega ; g'
      DO iir=1,nir
        PRINT *,qextaer(iir),omegaer(iir),gaer(iir)
      ENDDO
c



          DO iir=1,5
            caer(iir,1)=qextaer(1)*(1.E+0 - omegaer(1))
          ENDDO
          print *,'dans le co2 on prend Qabs : caer=',caer(1,1)



      RETURN
      END
